#!/usr/bin/env python
# encoding: utf-8
import sys
import time

import click
from click import style as st
from click.exceptions import Abort as AbortException

from .battleship import Game
from .exceptions import InvalidCoordinate, InvalidFormat
from .language import language
from .robot import Brainiac
from .version import __version__

from . import printer
_ = language.gettext


def player_shot(board_opponent, player, robot=None):
    if robot:
        x, y = robot.shot()
    else:
        result = click.prompt(_(
            'Choose your coordinate using one LETTER from A to Q and one NUMBER from 0 to '
            '16. \n'
            'Ex: a1, b15, c10 and etc... \n\nCoordinate: '
        ))
        x, y = result[0], result[1:]

    try:
        hit, ship = board_opponent.play(str(x), str(y))
    except (InvalidFormat, InvalidCoordinate) as exc:
        click.clear()
        printer.print_error_message(_(str(exc)))
        return False

    # passing the shot status to Robot learn
    if robot:
        robot.shot_status(x, y, ship)

    if hit:
        printer.print_ship_hit(ship, player)
        return True

    # Check if Wins
    if board_opponent.is_finished():
        sys.exit(printer.print_status(board_opponent, win=True))

    return True


@click.group()
@click.version_option(version=__version__, message='Py-Battleship, version %(version)s')
def main():
    """Py-Battleship.

    Is a simple Battleship game in text mode. Lets play!

    \b
    Example:
    \b
    \t $ py-battleship play
    """


@main.command('play')
def play():
    """Start Game
    """
    click.clear()
    player1_name = click.prompt(_('Player1  name: '))

    click.clear()
    players = click.prompt(_('Choose one or two players: '), type=click.Choice(['1', '2']))

    player2_name = 'CPU'
    if players == 2:
        player2_name = click.prompt(_('Player1  name: '))

    click.clear()
    player1_auto_build = click.prompt(_(
        'Do you want place your ships your self or \n'
        'you prefer to let automatic.\n\n'
        '1- Automatic \n'
        '0 - Place Own \n'
        'Choose: '), type=click.Choice(['0', '1'])
    )

    game_board1 = Game(
        player=player1_name,
        ships_visible=True
    )

    if bool(player1_auto_build):
        game_board1.auto_build_fleet()

    player2_autobuild = '1'
    if players == 2:
        player2_autobuild = click.prompt(_(
            'Do you want place your ships your self or \n'
            'you prefer to let automatic.\n\n'
            '1 - Automatic \n'
            '0 - Place Own \n'
            'Choose: '), type=click.Choice(['0', '1'])
        )

    game_board2 = Game(
        player=player2_name,
        ships_visible=False
    )
    if bool(player2_autobuild):
        game_board2.auto_build_fleet()

    try:
        while True:
            while True:
                printer.print_board(game_board1, game_board2)
                click.echo(st(_('Press CTRL+C to exit ...')))

                # Player1 Move against Player2
                if not player_shot(board_opponent=game_board2, player=1):
                    break

                # Check if Player1 Wins
                if game_board2.is_finished:
                    printer.print_status(game_board2, win=True)

                printer.print_board(game_board1, game_board2)

                # Player 2 shot
                robot = None
                if game_board2.player == 'CPU':
                    robot = Brainiac(game_board2)
                    click.echo('Player 2 is choosing a coordinate.')
                    time.sleep(2)

                # Player2 Move against Player1
                if not player_shot(board_opponent=game_board1, player=2, robot=robot):
                    break

                # Check if Player2 Wins
                if game_board1.is_finished:
                    printer.print_status(game_board1, win=False)

            printer.print_board(game_board1, game_board2)

    except (KeyboardInterrupt, AbortException):
        # TODO: Show statistics of the two player, for now is showing only for player1
        sys.exit(printer.print_status(game_board2))



import random

from .battleship import Game
from .constants import LETTERS


class Brainiac:
    """Decision Robot
    TODO: Change when hit a Ship
    """
    shot_directions = ('l', 'r', 'd', 'u')

    def __init__(self, player_board):
        self.player_board = player_board
        self.matrix = self._clean_board()
        self.selected = []
        self.shoots = []
        self.last_shot_direction = ''
        self.current_direction = 'l'

    def _clean_board(self):
        return [[None for _ in range(Game.COLS)] for _ in range(Game.ROWS)]

    def _next_search_direction(self):
        direction = self.shot_directions
        self.current_direction = direction[direction.index(self.current_direction) + 1]

    def _ship_hits(self, ship):
        hits = []
        for i_row, row in enumerate(self.matrix):
            for i_col, col in enumerate(row):
                hits.append((i_row, i_col)) if col == ship else None
        return hits

    def _get_random_shot(self):
        x = random.choice(list(LETTERS.keys())[:17])
        y = random.randint(0, Game.ROWS - 1)
        if (x, y) in self.selected:
            self._get_random_shot()

        self.selected.append((x, y))
        return x, y

    def _get_near_shot(self):
        *coordinates, ship = self.get_last_shot().values()

        if not ship:
            *coordinates, ship = self.get_before_last_shot().values()
            self._next_search_direction()

        x, y = coordinates
        if self.current_direction == 'l':
            coordinates = x + 1, y
        elif self.current_direction == 'r':
            coordinates = x - 1, y
        elif self.current_direction == 'd':
            coordinates = x, y + 1
        elif self.current_direction == 'u':
            coordinates = x, y - 1

        return coordinates

    def get_last_shot(self):
        return self.shoots[-1] if len(self.shoots) else {}

    def get_before_last_shot(self):
        return self.shoots[-2] if len(self.shoots) > 1 else {}

    def shot(self):
        last_shot = self.get_last_shot()
        before_last_shot = self.get_before_last_shot()
        if not last_shot.get('ship', '') and not before_last_shot.get('ship', ''):
            return self._get_random_shot()

        return self._get_near_shot()

    def shot_status(self, raw_x, raw_y, ship):
        x = int(LETTERS[raw_x.lower()])
        y = int(raw_y)

        self.shoots.append({
            'x': x,
            'y': y,
            'ship': ship
        })
        self.matrix[x][y] = ship
